<?php

require_once('NodeRPC_Service_Exception.php');

abstract class NodeRPC_Service_Abstract {

    /**
     * @var string
     */
    private $host;

    /**
     * @var int
     */
    private $port;

    /**
     * @var stdClass
     */
    protected $serviceDefinition;

    /**
     * @var string
     */
    protected $serviceName;

    public function __construct($host, $port = 80) {
        $this->host = $host;
        $this->port = $port;
        $this->init();
    }

    /**
     * Initialize service
     *
     * @abstract
     * @return void
     */
    abstract function init();

    /**
     * Convert arguments to http format
     *
     * @param  $args
     * @param  $paramDefinition
     * @return void
     */
    protected function marshalParams($args, $paramDefinition) {
        $params = array();
        foreach ($args as $i => &$arg) {
            $params[$paramDefinition[$i]['name']] = $arg;
        }
        return $params;
    }

    /**
     * Make http request with HttpRequest Object
     *
     * @throws NodeRPC_Service_Exception
     * @param  $url
     * @return mixed
     */
    private function httpHttpRequest($url, $params) {
        $httpRequest = new HttpRequest($url, HttpRequest::METH_POST);
        $httpRequest->addHeaders(array(
            'Content-type' => 'application/json'
        ));
        $httpRequest->setBody(json_encode($params));
        $httpRequest->send();
        
        if ($httpRequest->getResponseCode() == 200) {
            return json_decode($httpRequest->getResponseBody());
        }
        else {
            throw new NodeRPC_Service_Exception($httpRequest->getResponseBody());
        }
    }

    /**
     * Make http request with file_get_contents
     *
     * @throws Exception
     * @param  $url
     * @return mixed
     */
    private function httpFileGetContents($url, $params) {
        $opts = array('http' =>
            array(
                'method'  => 'POST',
                'header'  => 'Content-type: application/json',
                'content' => json_encode($params)
            )
        );
        $context  = stream_context_create($opts);

        $response = @file_get_contents($url, false, $context);
        if (false === $response) {
            throw new NodeRPC_Service_Exception("Request failed: " . $url);
        }

        if (strpos($http_response_header[0], '200 OK') !== false) {
            return json_decode($response, true);
        }
        else {
            throw new NodeRPC_Service_Exception($response);
        }
    }

    /**
     * Send request via http
     * @param  $method
     * @param  $args
     * @return void
     */
    protected function proxyCall($method, $args) {
        $params = $this->marshalParams($args, $this->serviceDefinition[$method]['params']);
        $route = "http://" . $this->host . ':' . $this->port . '/' . $this->serviceName . "/" . $method;
        if (class_exists('HttpRequest')) {
            return $this->httpHttpRequest($route, $params);
        }
        else {
            return $this->httpFileGetContents($route, $params);
        }
    }

}