var Proxy = require('noderpc/proxy').Abstract,
    rpcMarshal = require('noderpc/marshal'),
    util = require('util'),
    logger = require('logger').getLogger('noderpc');

function HttpProxy(service, host, port) {
    Proxy.call(this, service);
    this.host = host;
    if (port == undefined) {
        port = 80;
    }
    this.port = port;
    this.httpClient = require('http');
}

util.inherits(HttpProxy, Proxy);

HttpProxy.prototype.setHttpClient = function(httpClient) {
    this.httpClient = httpClient;
}

HttpProxy.prototype.buildRoute = function(method, args) {
    return "/" + this.serviceDefinition.name + "/" + method;
}

HttpProxy.prototype.proxyCall = function(method, args) {
    var argsCollection = this.spliceArgs(method, args),
        httpParams = rpcMarshal.toHttpParams(argsCollection.args, this.serviceDefinition.methods[method].params),
        options = {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            host: this.host,
            port: this.port,
            path: this.buildRoute(method, argsCollection.args)
        };

    // Execute http request
    var that = this;
    var req = this.httpClient.request(options,
        function(res) {
            var data = '';
            res.on('data', function(chunk) {
                data += chunk;
            });
            res.on('end', function() {
                if (res.statusCode !== 200) {
                    argsCollection.errorCallback(new Error(data));
                }
                else {
                    try {
                        data = data == '' ? undefined : JSON.parse(data);
                    }
                    catch (e) {
                        logger.warn("JSON Parse error on data: '" + data + "'");
                    }
                    argsCollection.returnCallback(data);
                }
            });
        }).on('error', function(e) {
            this.abort();
            argsCollection.errorCallback(e);
            that.emit('error', e, that);
        });
    req.write(JSON.stringify(httpParams));
    req.end();
}

module.exports = function(service, host, port) {
    return new HttpProxy(service, host, port);
}
