var util = require('util');
var shardProxy = require('noderpc/proxy/shard');
var localProxy = require('noderpc/proxy/local');

var fixture = require('test/fixtures/noderpc');

exports["unshardable"] = function (test) {

    var shard1 = localProxy(fixture.serviceShardInt, fixture.proxiedShardInt);
    var shard2 = localProxy(fixture.serviceShardInt, fixture.proxiedShardInt2);

    var testService = shardProxy(fixture.serviceShardInt, 'arg');
    testService.addShard(shard1);

    test.throws(function () {
        testService.foo('arg1', 'arg2', function() {
        }, function (error) {
        });
    });

    test.done();
}

exports["shardByInt"] = function (test) {

    var shard1 = localProxy(fixture.serviceShardInt, fixture.proxiedShardInt);
    var shard2 = localProxy(fixture.serviceShardInt, fixture.proxiedShardInt2);

    var testService = shardProxy(fixture.serviceShardInt, 'arg1');
    testService.addShard(shard1);
    testService.addShard(shard2);
    var myResult;
    testService.foo(1, 2);
    testService.foo(1, 2, function(result) {
        myResult = result;
    }, function(error) {
        throw error;
    });

    test.equals(1, myResult);

    testService.foo(4, 2, function(result) {
        myResult = result;
    });

    test.equals(0, myResult);

    testService.foo2(1);

    test.done();
}

exports["stringHash"] = function (test) {
    var testService = shardProxy(fixture.serviceShardInt, 'arg1');

    test.equals(0, testService.hashString('foo') % 2);
    test.equals(1, testService.hashString('1') % 2);

    test.done();
}

exports["shardByString"] = function (test) {

    var shard1 = localProxy(fixture.serviceShardString, fixture.proxiedShardString);
    var shard2 = localProxy(fixture.serviceShardString, fixture.proxiedShardString2);

    var testService = shardProxy(fixture.serviceShardString, 'arg1');
    testService.addShard(shard1);
    testService.addShard(shard2);
    var myResult;
    testService.foo("foo", 2);
    testService.foo("foo", 2, function(result) {
        myResult = result;
    }, function(error) {
        throw error;
    });

    test.equals(1, myResult);

    testService.foo("1", 2, function(result) {
        myResult = result;
    });

    test.equals(2, myResult);

    test.done();
}